      SUBROUTINE NERS_COMP_SPL ( NERS, IUER )
! ************************************************************************
! *                                                                      *
! *   Routine  NERS_COMP_SPL
! *                                                                      *
! * ### 18-JUN-2016   NERS_COMP_SPL   v1.0 (c) L. Petrov 18-JUN-2016 ### *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'astro_constants.i'
      INCLUDE   'ners.i'
      INTEGER*4  IUER
      TYPE     ( NERS__TYPE ) :: NERS
      REAL*8     TIM_EPS
      PARAMETER  ( TIM_EPS = 1.0D0 ) 
      REAL*8,    ALLOCATABLE :: TEMP_ARR(:)
      REAL*8     TIM_BEG, TIM_END, TIM_STEP, ARG_STEP, DEPS, DPSI, &
     &           EVEC(3,0:2), HEO_VEC(3,0:2), MAT_ROT(3,3,0:2), &
     &           DER_EOP(2,NERS__MEL), UTC_CUR, UTC_M_TAI, DPSI_RATE, &
     &           DEPS_RATE, SAVE_TAI_GEN
      REAL*8     CNS_VAL_SIG, CNS_DER_SIG, CNS_DR2_SIG, ARG_EPS
      PARAMETER  ( CNS_VAL_SIG = 0.0D0 )
      PARAMETER  ( CNS_DER_SIG = 0.0D0 )
      PARAMETER  ( CNS_DR2_SIG = 0.0D0 )
      PARAMETER  ( ARG_EPS     = 1.D-8 )
      CHARACTER  STR*128, STR1*21, STR2*21
      INTEGER*8  SIZE_I8
      INTEGER*4  J1, J2, J3, IND, IS, UNIX_DATE, IER
      INTEGER*4, EXTERNAL :: TIME, FILE_INFO
      CHARACTER, EXTERNAL :: TIM_TO_DATE*23
!
      IF ( NERS%FCS_STATUS .NE. NERS__LOAD ) THEN
           CALL ERR_LOG ( 4711, IUER, 'NERS_COMP_SPL', 'Trap of '// &
     &         'internal control: NERS was not loaded. Please execute '// &
     &         'routine ners_load first' )
           RETURN 
      END IF
!
      IS = FILE_INFO ( TRIM(NERS%CNF%FCS_FILE)//CHAR(0), UNIX_DATE, &
     &                 SIZE_I8 )
      IF ( NERS%CNF%AGE_SPL > 0.0D0                            .AND. &
     &     ( TIME ( %VAL(0) ) - UNIX_DATE ) > NERS%CNF%AGE_SPL       ) THEN
!
! -------- Spline is too old. We need to re-create it
!
           SAVE_TAI_GEN = NERS%FCS%TAI_GEN
           IF ( ( TIME ( %VAL(0) ) - UNIX_DATE ) > NERS%CNF%AGE_FCS ) THEN
!
! ------------- EOP forecast file is too old
!
                IER = IUER
                CALL NERS_FETCH ( NERS, IER )
                IF ( IER .NE. 0 ) THEN
                     CALL ERR_LOG ( 4711, IUER, 'NERS_COMP_SPL', 'Error in '// &
     &                   'an attempt to retrieve NERS forecast parameters '// &
     &                   'form the remote server' )
                     RETURN 
               END IF
           END IF
!
! -------- Clean forecast related elements of the NERS internal data strcuture
!
           CALL NERS_QUIT ( NERS__FCS, NERS )
!
! -------- Load forecast file
!
           IER = IUER
           CALL NERS_LOAD ( NERS, IER )
           IF ( IER .NE. 0 ) THEN
                CALL ERR_LOG ( 4712, IUER, 'NERS_COMP_SPL', 'Error in '// &
     &              'an attempt to retrieve NERS forecast parameters '// &
     &              'form the remote server' )
                RETURN 
           END IF
      END IF
!
! --- Clean Bspline-related elements of the NERS internal data strcuture
!
      CALL NERS_QUIT ( NERS__EXP, NERS )
!
      TIM_BEG = 0.0D0
      TIM_END = NERS%TIM_STOP - NERS%TIM_START
      IF ( TIM_END < 0.0D0 ) THEN
           TIM_END = NERS%FCS%ARG_3(NERS%FCS%NK_3)
           TIM_BEG = NERS%FCS%ARG_3(NERS%FCS%NK_3) - NERS__INTR_MAX
      END IF
      IF ( TIM_BEG < TIM_END - NERS__INTR_MAX ) THEN
           CALL CLRCH ( STR  )
           CALL CLRCH ( STR1 )
           CALL CLRCH ( STR2 )
           WRITE ( UNIT=STR(1:10), FMT='(F10.4)' ) NERS__INTR_MAX
           STR1 = TIM_TO_DATE ( TIM_BEG, IER )
           STR2 = TIM_TO_DATE ( TIM_END, IER )
           CALL ERR_LOG ( 4713, IUER, 'NERS_COMP_SPL', 'Trap of '// &
     &         'internal control: the interval for EOP spline '// &
     &         'expansion is too large: ['//STR1//', '//STR2//'] -- '// &
     &         'longer than '//TRIM(STR)//' days' )
           RETURN 
      END IF
! 
      NERS%EXP%L_NOD = (TIM_END - TIM_BEG)/NERS__TIM_STEP
      IF ( TIM_END - TIM_BEG >  NERS%EXP%L_NOD*NERS__TIM_STEP + TIM_EPS ) THEN
           NERS%EXP%L_NOD = NERS%EXP%L_NOD + 1
      END IF
      IF ( NERS%EXP%L_NOD < NERS__MIN_NODES ) THEN
           NERS%EXP%L_NOD = NERS__MIN_NODES 
      END IF
      ARG_STEP = (TIM_END - TIM_BEG)/(NERS%EXP%L_NOD - 1)
      NERS%EXP%L_TIM = NERS%EXP%L_NOD + 2
      TIM_STEP = (TIM_END - TIM_BEG)/(NERS%EXP%L_TIM - 1)
!
      IF ( ASSOCIATED ( NERS%EXP%ARG ) ) DEALLOCATE ( NERS%EXP%ARG )
      ALLOCATE ( NERS%EXP%ARG(NERS%EXP%L_NOD), STAT=IER )
      IF ( IER .NE.0 ) THEN
           CALL ERR_LOG ( 4714, IUER, 'NERS_COMP_SPL', 'Error in attempt '// &
     &         'to allocate dynamic memory for array NERS%EXP%ARG' )
           RETURN 
      END IF
!
      IF ( ASSOCIATED ( NERS%EXP%BSPL ) ) DEALLOCATE ( NERS%EXP%BSPL )
      ALLOCATE ( NERS%EXP%BSPL(1-NERS__MDEG:NERS%EXP%L_NOD-1,NERS__MEL), STAT=IER )
      IF ( IER .NE.0 ) THEN
           CALL ERR_LOG ( 4715, IUER, 'NERS_COMP_SPL', 'Error in attempt '// &
     &         'to allocate dynamic memory for array NERS%EXP%BSPL' )
           RETURN 
      END IF
!
      IF ( ASSOCIATED ( NERS%EXP%TIM ) ) DEALLOCATE ( NERS%EXP%TIM )
      ALLOCATE ( NERS%EXP%TIM(NERS%EXP%L_TIM), STAT=IER )
      IF ( IER .NE.0 ) THEN
           CALL ERR_LOG ( 4716, IUER, 'NERS_COMP_SPL', 'Error in attempt '// &
     &         'to allocate dynamic memory for array NERS%EXP%TIM' )
           RETURN 
      END IF
!
      IF ( ASSOCIATED ( NERS%EXP%VAL ) ) DEALLOCATE ( NERS%EXP%VAL )
      ALLOCATE ( NERS%EXP%VAL(NERS%EXP%L_TIM,NERS__MEL), STAT=IER )
      IF ( IER .NE.0 ) THEN
           CALL ERR_LOG ( 4717, IUER, 'NERS_COMP_SPL', 'Error in attempt '// &
     &         'to allocate dynamic memory for array NERS%EXP%VAL' )
           RETURN 
      END IF
      NERS%EXP_STATUS = NERS__ALLC 
!
      DO 410 J1=1,NERS%EXP%L_TIM
         NERS%EXP%TIM(J1) = TIM_BEG + (J1-1)*TIM_STEP
         IER = IUER
         CALL NERS_GET_EVEC ( NERS, NERS%TIM_START + NERS%EXP%TIM(J1), EVEC, IER )
         IF ( IER .NE. 0 ) THEN
              CALL ERR_LOG ( 4718, IUER, 'NERS_COMP_SPL', 'Error in '// &
     &            'an attempt to compute the EOP vector on the '// &
     &            'requested moment of time' )
              RETURN 
         END IF
!
         IER = IUER
         CALL NERS_GET_HEO ( NERS, NERS%TIM_START + NERS%EXP%TIM(J1), &
     &                       EVEC(3,0)/UT1__TO__E3, DPSI, DEPS, HEO_VEC, &
     &                       DPSI_RATE, DEPS_RATE, IER )
         IF ( IER .NE. 0 ) THEN
              CALL ERR_LOG ( 4719, IUER, 'NERS_COMP_SPL', 'Error in '// &
     &            'an attempt to compute the contribution of '// &
     &            'harmonic EOP variation of teh requested '// &
     &            'moment of time' )
              RETURN 
         END IF
!
         IER = IUER
         CALL NERS_GET_MATROT ( NERS, NERS%TIM_START + NERS%EXP%TIM(J1), MAT_ROT, IER )
         IF ( IER .NE. 0 ) THEN
              CALL ERR_LOG ( 4720, IUER, 'NERS_COMP_SPL', 'Error in '// &
     &            'computing the rotation matrix for Earth orientation' ) 
              RETURN 
         END IF
!         
         NERS%EXP%VAL(J1,NERS__E1) = EVEC(1,0)
         NERS%EXP%VAL(J1,NERS__E2) = EVEC(2,0)
         NERS%EXP%VAL(J1,NERS__E3) = EVEC(3,0)
         NERS%EXP%VAL(J1,NERS__H1) = HEO_VEC(1,0)
         NERS%EXP%VAL(J1,NERS__H2) = HEO_VEC(2,0)
         NERS%EXP%VAL(J1,NERS__H3) = HEO_VEC(3,0)
         NERS%EXP%VAL(J1,NERS__DP) = DPSI
         NERS%EXP%VAL(J1,NERS__DE) = DEPS
         NERS%EXP%VAL(J1,NERS__MAT11) = MAT_ROT(1,1,0)
         NERS%EXP%VAL(J1,NERS__MAT21) = MAT_ROT(2,1,0)
         NERS%EXP%VAL(J1,NERS__MAT31) = MAT_ROT(3,1,0)
         NERS%EXP%VAL(J1,NERS__MAT12) = MAT_ROT(1,2,0)
         NERS%EXP%VAL(J1,NERS__MAT22) = MAT_ROT(2,2,0)
         NERS%EXP%VAL(J1,NERS__MAT32) = MAT_ROT(3,2,0)
         NERS%EXP%VAL(J1,NERS__MAT13) = MAT_ROT(1,3,0)
         NERS%EXP%VAL(J1,NERS__MAT23) = MAT_ROT(2,3,0)
         NERS%EXP%VAL(J1,NERS__MAT33) = MAT_ROT(3,3,0)
 410  CONTINUE 
!
      DO 420 J2=1,NERS%EXP%L_NOD
         NERS%EXP%ARG(J2) = TIM_BEG + (J2-1)*ARG_STEP
 420  CONTINUE 
      NERS%EXP%ARG(1)              = NERS%EXP%ARG(1)              - ARG_EPS
      NERS%EXP%ARG(NERS%EXP%L_NOD) = NERS%EXP%ARG(NERS%EXP%L_NOD) + ARG_EPS
!
      CALL ERR_PASS ( IUER, IER )
      CALL EBSPL_LSQ_CNS3_VEC ( NERS%EXP%L_TIM, NERS__MEL,  NERS%EXP%TIM, NERS%EXP%VAL,  &
     &                          NERS%EXP%L_NOD, NERS__MDEG, NERS%EXP%ARG, NERS%EXP%BSPL, &
     &                          CNS_VAL_SIG, CNS_DER_SIG, CNS_DR2_SIG, IER )
      IF ( IER .NE. 0 ) THEN
           CALL ERR_LOG ( 4721, IUER, 'NERS_LOAD', 'Error in computing '// &
     &         'the coefficients of B-spline expansion' )
           RETURN 
      END IF
!
      UTC_CUR = TIME ( %VAL(0) ) - UNIX__J2000_UTC
!
      CALL ERR_PASS   ( IUER, IER )
      CALL NERS_GET_UTCMTAI ( NERS, UTC_CUR, UTC_M_TAI, IER )
      IF ( IER .NE. 0 ) THEN
           CALL ERR_LOG ( 4722, IUER, 'NERS_LOAD', 'Error in getting '// &
     &         'UTC minus TAI difference' )
           RETURN 
      END IF
      IF ( ASSOCIATED ( NERS%EXP%VAL ) ) DEALLOCATE ( NERS%EXP%VAL )
      IF ( ASSOCIATED ( NERS%EXP%TIM ) ) DEALLOCATE ( NERS%EXP%TIM )
      NERS%EXP_STATUS = NERS__COMP
!
      CALL ERR_LOG ( 0, IUER ) 
      RETURN
      END  SUBROUTINE  NERS_COMP_SPL  !#!#
