      SUBROUTINE NERS_GET_EOP ( NERS, TIM_TAI, CPAR, M_PAR, L_PAR, PARS, IUER )
! ************************************************************************
! *                                                                      *
! *   Routine  NERS_GET_EOP
! *                                                                      *
! *  ### 16-JUN-2016  NERS_GET_EOP  v1.2 (c) L. Petrov  17-NOV-2016 ###  *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'ners.i'
      INCLUDE   'astro_constants.i'
      TYPE     ( NERS__TYPE ) :: NERS
      INTEGER*4  M_PAR, L_PAR, IUER 
      CHARACTER  CPAR*(*)
      REAL*8     PARS(M_PAR), TIM_TAI, EVEC(3,0:2), UTC_CUR, TAI_CUR, &
     &           TIM_TAI_USED, UTC_M_TAI, DPSI, DEPS, HEO_APS_VEC(3,0:2), &
     &           MAT_ROT(3,3,0:2), DPSI_RATE, DEPS_RATE
      REAL*8       LOD__TO__ER3
      PARAMETER  ( LOD__TO__ER3 =  1.00273781191135448E0*OM__EAR**2/PI2 )
      INTEGER*4  IS, UNIX_DATE, IER
      INTEGER*8  SIZE_I8
      INTEGER*4, EXTERNAL :: ILEN, I_LEN, TIME, FILE_INFO
!
      UTC_CUR = TIME ( %VAL(0) ) - UNIX__J2000_UTC
!
      IF ( NERS%FCS_STATUS .NE. NERS__LOAD .OR. &
     &     (UTC_CUR - NERS%UTC_LOAD) > NERS__AGE_MIN ) THEN
           IF ( NERS%FCS_STATUS .NE. NERS__INIT ) THEN
                CALL ERR_LOG ( 4311, IUER, 'NERS_GET_EOP', 'NERS data '// &
     &              'structure has not been initialized. Please run '// &
     &              'NERS_INIT first' )
                RETURN 
           END IF
!
           IS = FILE_INFO ( TRIM(NERS%CNF%FCS_FILE)//CHAR(0), UNIX_DATE, &
     &                      SIZE_I8 )
           IF ( IS .NE. 0 .OR. (TIME(%VAL(0)) - UNIX_DATE) > NERS%CNF%AGE_FCS ) THEN
                IER = IUER
                CALL NERS_FETCH ( NERS, IER )
                IF ( IER .NE. 0 ) THEN
                     CALL ERR_LOG ( 4312, IUER, 'NERS_GET_EOP', 'Error in '// &
     &                   'an attempt to retrieve NERS forecast parameters '// &
     &                   'form the remote server' )
                     RETURN 
                END IF
           END IF
!
           IER = IUER
           CALL NERS_LOAD ( NERS, IER )
           IF ( IER .NE. 0 ) THEN
                CALL ERR_LOG ( 4313, IUER, 'NERS_GET_EOP', 'Error in '// &
     &              'an attempt to retrieve NERS forecast parameters '// &
     &              'form the remote server' )
                RETURN 
           END IF
      END IF
!
      IF ( TIM_TAI == -1.0D0 ) THEN
           CALL NERS_GET_UTCMTAI ( NERS, UTC_CUR, UTC_M_TAI, IER )
           TIM_TAI_USED = UTC_CUR - UTC_M_TAI
         ELSE 
           TIM_TAI_USED = TIM_TAI
      END IF
!
      IF ( CPAR == 'utcmtai' ) THEN
           IER = IUER
           CALL NERS_GET_UTCMTAI ( NERS, TIM_TAI_USED, UTC_M_TAI, IER )
           IF ( IER .NE. 0 ) THEN
                CALL ERR_LOG ( 4314, IUER, 'NERS_GET_EOP', 'Error in '// &
     &              'an attempt to compute UTC minus TAI function on '// &
     &              'the requested moment of time' )
                RETURN 
           END IF
           L_PAR = 1
           PARS(1) = UTC_M_TAI
           CALL ERR_LOG ( 0, IUER )
           RETURN 
      END IF
!
      IF ( NERS%TIM_START == NERS%TIM_STOP ) THEN
           IF ( CPAR == 'mat'    .OR. &
     &          CPAR == 'matr'   .OR. &
     &          CPAR == 'matrr'  .OR. &
     &          CPAR == 'matall'      ) THEN
!
                IER = IUER
                CALL NERS_GET_MATROT ( NERS, TIM_TAI_USED, MAT_ROT, IER )
                IF ( IER .NE. 0 ) THEN
                     CALL ERR_LOG ( 4316, IUER, 'NERS_GET_EOP', 'Error in '// &
     &                   'computing the rotatgion matrix for Earth orientation' ) 
                     RETURN 
                END IF
              ELSE
                IER = IUER
                CALL NERS_GET_EVEC ( NERS, TIM_TAI_USED, EVEC, IER )
                IF ( IER .NE. 0 ) THEN
                     CALL ERR_LOG ( 4317, IUER, 'NERS_GET_EOP', 'Error in '// &
     &                   'an attempt to compute the EOP vector on the '// &
     &                   'requested moment of time' )
                     RETURN 
                END IF
!
                IF ( CPAR == 'dpsi'  .OR. &
     &               CPAR == 'deps'  .OR. &
     &               CPAR == 'dpsir' .OR. &
     &               CPAR == 'depsr' .OR. &
     &               CPAR == 'eops'  .OR. &
     &               CPAR == 'nut'   .OR. &
     &               CPAR == 'nutr'  .OR. &
     &               CPAR == 'heo'   .OR. &
     &               CPAR == 'h1'    .OR. &
     &               CPAR == 'h2'    .OR. &
     &               CPAR == 'h3'    .OR. &
     &               CPAR == 'h1r'   .OR. &
     &               CPAR == 'h2r'   .OR. &
     &               CPAR == 'h3r'   .OR. &
     &               CPAR == 'h1rr'  .OR. &
     &               CPAR == 'h2rr'  .OR. &
     &               CPAR == 'h3rr'       ) THEN
!
                     IER = -1
                     CALL NERS_GET_HEO ( NERS, TIM_TAI_USED, EVEC(3,0)/UT1__TO__E3, &
     &                                   DPSI, DEPS, HEO_APS_VEC, &
     &                                   DPSI_RATE, DEPS_RATE, IER )
                     IF ( IER .NE. 0 ) THEN
                          CALL ERR_LOG ( 4318, IUER, 'NERS_GET_EOP', 'Error in '// &
     &                        'an attempt to compute the contribution of '// &
     &                        'harmonic EOP variation of the requested '// &
     &                        'moment of time' )
                          RETURN 
                     END IF
                   ELSE
                     IF ( CPAR == 'e1'   .OR. CPAR == 'e2'   .OR. CPAR == 'e3'          .OR. &
     &                    CPAR == 'e1r'  .OR. CPAR == 'e2r'  .OR. CPAR == 'e3r'         .OR. &
     &                    CPAR == 'e1rr' .OR. CPAR == 'e2rr' .OR. CPAR == 'e3rr'        .OR. &
     &                    CPAR == 'ut1mtai' .OR. CPAR == 'ut1rat'                       .OR. &
     &                    CPAR == 'ut1rr'   .OR. CPAR == 'lod'                          .OR. &
     &                    CPAR == 'eop3'    .OR. CPAR == 'eop3r'                        .OR. &
     &                    CPAR == 'xpol'    .OR. CPAR == 'ypol'                         .OR. &
     &                    CPAR == 'xpolr'   .OR. CPAR == 'ypolr'                        .OR. &
     &                    CPAR == 'xpolrr'  .OR. CPAR == 'ypolrr' .OR. CPAR == 'utcmtai'     ) THEN
                          CONTINUE 
                        ELSE
                          CALL ERR_LOG ( 4319, IUER, 'NERS_GET_EOP', 'EOP parameter '// &
     &                         TRIM(CPAR)//' is not supported' )
                          RETURN 
                     END IF
                END IF
           END IF
!
           IF ( CPAR == 'e1' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,0)
              ELSE IF ( CPAR == 'e2' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,0)
              ELSE IF ( CPAR == 'e3' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,0)
              ELSE IF ( CPAR == 'e1r' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,1)
              ELSE IF ( CPAR == 'e2r' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,1)
              ELSE IF ( CPAR == 'e3r' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,1)
              ELSE IF ( CPAR == 'e1rr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,2)
              ELSE IF ( CPAR == 'e2rr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,2)
              ELSE IF ( CPAR == 'e3rr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,2)
              ELSE IF ( CPAR == 'ut1mtai' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,0)/UT1__TO__E3
              ELSE IF ( CPAR == 'ut1rat' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,1)/UT1__TO__E3
              ELSE IF ( CPAR == 'ut1rr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,2)/UT1__TO__E3
              ELSE IF ( CPAR == 'lod' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(3,1)/LOD__TO__ER3
              ELSE IF ( CPAR == 'xpol' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,0)*RAD__TO__ARCSEC
              ELSE IF ( CPAR == 'ypol' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,0)*RAD__TO__ARCSEC
              ELSE IF ( CPAR == 'xpolr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,1)*RAD__TO__ARCSEC*86400.0D0
              ELSE IF ( CPAR == 'ypolr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,1)*RAD__TO__ARCSEC*86400.0D0
              ELSE IF ( CPAR == 'xpolrr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(2,2)*RAD__TO__ARCSEC*86400.0D0**2
              ELSE IF ( CPAR == 'ypolrr' ) THEN
                L_PAR = 1
                PARS(1) = EVEC(1,2)*RAD__TO__ARCSEC*86400.0D0**2
              ELSE IF ( CPAR == 'dpsi' ) THEN
                L_PAR = 1
                PARS(1) = DPSI
              ELSE IF ( CPAR == 'deps' ) THEN
                L_PAR = 1
                PARS(1) = DEPS
              ELSE IF ( CPAR == 'dpsir' ) THEN
                L_PAR = 1
                PARS(1) = DPSI_RATE
              ELSE IF ( CPAR == 'depsr' ) THEN
                L_PAR = 1
                PARS(1) = DEPS_RATE
              ELSE IF ( CPAR == 'nut' ) THEN
                L_PAR = 2
                PARS(1) = DPSI
                PARS(2) = DEPS
              ELSE IF ( CPAR == 'nutr' ) THEN
                L_PAR = 4
                PARS(1) = DPSI
                PARS(2) = DEPS
                PARS(3) = DPSI_RATE
                PARS(4) = DEPS_RATE
              ELSE IF ( CPAR == 'heo' ) THEN
                L_PAR = 3
                PARS(1) = HEO_APS_VEC(1,0)
                PARS(2) = HEO_APS_VEC(2,0)
                PARS(3) = HEO_APS_VEC(3,0)
              ELSE IF ( CPAR == 'heorr' ) THEN
                L_PAR = 9
                PARS(1) = HEO_APS_VEC(1,0)
                PARS(2) = HEO_APS_VEC(2,0)
                PARS(3) = HEO_APS_VEC(3,0)
                PARS(4) = HEO_APS_VEC(1,1)
                PARS(5) = HEO_APS_VEC(2,1)
                PARS(6) = HEO_APS_VEC(3,1)
                PARS(7) = HEO_APS_VEC(1,2)
                PARS(8) = HEO_APS_VEC(2,2)
                PARS(9) = HEO_APS_VEC(3,2)
              ELSE IF ( CPAR == 'h1' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(1,0)
              ELSE IF ( CPAR == 'h2' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(2,0)
              ELSE IF ( CPAR == 'h3' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(3,0)
              ELSE IF ( CPAR == 'h1r' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(1,1)
              ELSE IF ( CPAR == 'h2r' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(2,1)
              ELSE IF ( CPAR == 'h3r' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(3,1)
              ELSE IF ( CPAR == 'h1rr' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(1,2)
              ELSE IF ( CPAR == 'h2rr' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(2,2)
              ELSE IF ( CPAR == 'h3rr' ) THEN
                L_PAR = 1
                PARS(1) = HEO_APS_VEC(3,2)
              ELSE IF ( CPAR == 'mat' ) THEN
                L_PAR = 9
                CALL MEMCPY ( PARS, MAT_ROT(1,1,0), %VAL(9*8) )
              ELSE IF ( CPAR == 'matr' ) THEN
                L_PAR = 9
                CALL MEMCPY ( PARS, MAT_ROT(1,1,1), %VAL(9*8) )
              ELSE IF ( CPAR == 'matrr' ) THEN
                L_PAR = 9
                CALL MEMCPY ( PARS, MAT_ROT(1,1,2), %VAL(9*8) )
              ELSE IF ( CPAR == 'matall' ) THEN
                L_PAR = 27
                CALL MEMCPY ( PARS, MAT_ROT, %VAL(9*3*8) )
              ELSE IF ( CPAR == 'eop3' ) THEN
                L_PAR = 3
                PARS(1) = EVEC(2,0)*RAD__TO__ARCSEC
                PARS(2) = EVEC(1,0)*RAD__TO__ARCSEC
                PARS(3) = EVEC(3,0)/UT1__TO__E3
              ELSE IF ( CPAR == 'eop3r' ) THEN
                L_PAR = 6
                PARS(1) = EVEC(2,0)*RAD__TO__ARCSEC
                PARS(2) = EVEC(1,0)*RAD__TO__ARCSEC
                PARS(3) = EVEC(3,0)/UT1__TO__E3
                PARS(4) = EVEC(2,1)*RAD__TO__ARCSEC*86400.0D0
                PARS(5) = EVEC(1,1)*RAD__TO__ARCSEC*86400.0D0
                PARS(6) = EVEC(3,1)/UT1__TO__E3*86400.0D0
              ELSE IF ( CPAR == 'eops' ) THEN
                L_PAR = 8
                PARS(1) = EVEC(2,0)*RAD__TO__ARCSEC
                PARS(2) = EVEC(1,0)*RAD__TO__ARCSEC
                PARS(3) = EVEC(3,0)/UT1__TO__E3
                PARS(4) = EVEC(2,1)*RAD__TO__ARCSEC*86400.0D0
                PARS(5) = EVEC(1,1)*RAD__TO__ARCSEC*86400.0D0
                PARS(6) = EVEC(3,1)/UT1__TO__E3*86400.0D0
                PARS(7) = DPSI*RAD__TO__ARCSEC
                PARS(8) = DEPS*RAD__TO__ARCSEC
              ELSE
                CALL ERR_LOG ( 4320, IUER, 'NERS_GET_EOP', 'Earth orientation '// &
     &              'parameter name '//TRIM(CPAR)//' is not supported' )
                RETURN 
           END IF
         ELSE
           IF ( NERS%EXP_STATUS .NE. NERS__COMP .OR. &
     &          TIM_TAI_USED < NERS%TIM_START   .OR. &
     &          TIM_TAI_USED > NERS%TIM_STOP         ) THEN
!
                IER = IUER
                CALL NERS_COMP_SPL ( NERS, IER )
                IF ( IER .NE. 0 ) THEN
                     CALL ERR_LOG ( 4321, IUER, 'NERS_GET_EOP', 'Error in '// &
     &                   'computation of the spline coefficients' )
                     RETURN 
                END IF           
           END IF           
!
           IER = IUER
           CALL NERS_GET_SPL  ( NERS, TIM_TAI_USED, CPAR, L_PAR, PARS, IER )
           IF ( IER .NE. 0 ) THEN
                CALL ERR_LOG ( 4322, IUER, 'NERS_GET_EOP', 'Error during '// &
     &              'spline interpolation' )
                RETURN 
           END IF           
      END IF
!
      CALL ERR_LOG ( 0, IUER )
      RETURN
      END  SUBROUTINE  NERS_GET_EOP  !#!#
